package anywheresoftware.b4a.agraham.opengl2;

import java.lang.reflect.Method;
import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.graphics.Bitmap;
import android.graphics.PixelFormat;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.opengl.GLUtils;
import android.opengl.Matrix;
import android.opengl.Visibility;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.ActivityObject;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import anywheresoftware.b4a.objects.ViewWrapper;

/**
 *This object does all the drawing work and is an exposure of the Android GLES20 object.
 *
 *The GL2 function and parameter names mirror those of the GLES20 object which should be present on
 *Android versions 2.2 and later (API level 8 or later).
 *
 *Some variations of functions that use native buffers instead of arrays are omitted, generally
 *where there is an array equivalent. Where there is not an equivalent using an array has
 *been synthesised.
 */
@ShortName("GL2")
@ActivityObject
@Version(1.5f)
@Author("Andrew Graham")
public class GLWrapper2
{
	// 1.0 	initial release
	// 1.1	added glBufferDataF, glBufferSubDataF, added glBufferDataI, glBufferSubDataI, 
	// 1.2  changed and corrected glVertexAttribPointerF, added new glVertexAttribPointer
	// 1.3  added more Initialize methods to GLSurfaceViewWrapper2
	// 1.4	corrected typo of gActiveTexture(), now glActiveTexture() 
	//		added SetLookAtM to MatrixWrapper2
	// 1.5  added some more functions to deal with short and byte types.

	private static final double version = 1.5;

	/**
	 *Android includes support for high performance 3D graphics via the OpenGL ES API.
	 *OpenGL ES is a flavor of the OpenGL specification intended for embedded devices.
	 *Versions of OpenGL ES are loosely peered to versions of the primary OpenGL standard.
	 *Android currently supports OpenGL ES 2.0, which corresponds to OpenGL 1.5.
	 *If the application you have in mind is possible with OpenGL 1.5 on a desktop system,
	 *it should be possible on Android.
	 *
	 *This library also contains a Matrix2 object that operates on 4x4 OpenGL matrices and
	 *a Visibility2 object that implements a collection of utility methods for computing the visibility
	 *of triangle meshes.
	 */
	public void LIBRARY_DOC()
	{
	}

	// Private fields
	GL10 gl;

	public GLWrapper2(GL10 gl10)
	{
		gl = gl10;
	}

	/**
	 *Returns the version of the library.
	 */
	public double getVersion()
	{
		return version;
	}

	// GLES20 CONSTANTS

	public final int GL_ACTIVE_ATTRIBUTES = GLES20.GL_ACTIVE_ATTRIBUTES;
	public final int GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = GLES20.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH;
	public final int GL_ACTIVE_TEXTURE = GLES20.GL_ACTIVE_TEXTURE;
	public final int GL_ACTIVE_UNIFORMS = GLES20.GL_ACTIVE_UNIFORMS;
	public final int GL_ACTIVE_UNIFORM_MAX_LENGTH = GLES20.GL_ACTIVE_UNIFORM_MAX_LENGTH;
	public final int GL_ALIASED_LINE_WIDTH_RANGE = GLES20.GL_ALIASED_LINE_WIDTH_RANGE;
	public final int GL_ALIASED_POINT_SIZE_RANGE = GLES20.GL_ALIASED_POINT_SIZE_RANGE;
	public final int GL_ALPHA = GLES20.GL_ALPHA;
	public final int GL_ALPHA_BITS = GLES20.GL_ALPHA_BITS;
	public final int GL_ALWAYS = GLES20.GL_ALWAYS;
	public final int GL_ARRAY_BUFFER = GLES20.GL_ARRAY_BUFFER;
	public final int GL_ARRAY_BUFFER_BINDING = GLES20.GL_ARRAY_BUFFER_BINDING;
	public final int GL_ATTACHED_SHADERS = GLES20.GL_ATTACHED_SHADERS;
	public final int GL_BACK = GLES20.GL_BACK;
	public final int GL_BLEND = GLES20.GL_BLEND;
	public final int GL_BLEND_COLOR = GLES20.GL_BLEND_COLOR;
	public final int GL_BLEND_DST_ALPHA = GLES20.GL_BLEND_DST_ALPHA;
	public final int GL_BLEND_DST_RGB = GLES20.GL_BLEND_DST_RGB;
	public final int GL_BLEND_EQUATION = GLES20.GL_BLEND_EQUATION;
	public final int GL_BLEND_EQUATION_ALPHA = GLES20.GL_BLEND_EQUATION_ALPHA;
	public final int GL_BLEND_EQUATION_RGB = GLES20.GL_BLEND_EQUATION_RGB;
	public final int GL_BLEND_SRC_ALPHA = GLES20.GL_BLEND_SRC_ALPHA;
	public final int GL_BLEND_SRC_RGB = GLES20.GL_BLEND_SRC_RGB;
	public final int GL_BLUE_BITS = GLES20.GL_BLUE_BITS;
	public final int GL_BOOL = GLES20.GL_BOOL;
	public final int GL_BOOL_VEC2 = GLES20.GL_BOOL_VEC2;
	public final int GL_BOOL_VEC3 = GLES20.GL_BOOL_VEC3;
	public final int GL_BOOL_VEC4 = GLES20.GL_BOOL_VEC4;
	public final int GL_BUFFER_SIZE = GLES20.GL_BUFFER_SIZE;
	public final int GL_BUFFER_USAGE = GLES20.GL_BUFFER_USAGE;
	public final int GL_BYTE = GLES20.GL_BYTE;
	public final int GL_CCW = GLES20.GL_CCW;
	public final int GL_CLAMP_TO_EDGE = GLES20.GL_CLAMP_TO_EDGE;
	public final int GL_COLOR_ATTACHMENT0 = GLES20.GL_COLOR_ATTACHMENT0;
	public final int GL_COLOR_BUFFER_BIT = GLES20.GL_COLOR_BUFFER_BIT;
	public final int GL_COLOR_CLEAR_VALUE = GLES20.GL_COLOR_CLEAR_VALUE;
	public final int GL_COLOR_WRITEMASK = GLES20.GL_COLOR_WRITEMASK;
	public final int GL_COMPILE_STATUS = GLES20.GL_COMPILE_STATUS;
	public final int GL_COMPRESSED_TEXTURE_FORMATS = GLES20.GL_COMPRESSED_TEXTURE_FORMATS;
	public final int GL_CONSTANT_ALPHA = GLES20.GL_CONSTANT_ALPHA;
	public final int GL_CONSTANT_COLOR = GLES20.GL_CONSTANT_COLOR;
	public final int GL_CULL_FACE = GLES20.GL_CULL_FACE;
	public final int GL_CULL_FACE_MODE = GLES20.GL_CULL_FACE_MODE;
	public final int GL_CURRENT_PROGRAM = GLES20.GL_CURRENT_PROGRAM;
	public final int GL_CURRENT_VERTEX_ATTRIB = GLES20.GL_CURRENT_VERTEX_ATTRIB;
	public final int GL_CW = GLES20.GL_CW;
	public final int GL_DECR = GLES20.GL_DECR;
	public final int GL_DECR_WRAP = GLES20.GL_DECR_WRAP;
	public final int GL_DELETE_STATUS = GLES20.GL_DELETE_STATUS;
	public final int GL_DEPTH_ATTACHMENT = GLES20.GL_DEPTH_ATTACHMENT;
	public final int GL_DEPTH_BITS = GLES20.GL_DEPTH_BITS;
	public final int GL_DEPTH_BUFFER_BIT = GLES20.GL_DEPTH_BUFFER_BIT;
	public final int GL_DEPTH_CLEAR_VALUE = GLES20.GL_DEPTH_CLEAR_VALUE;
	public final int GL_DEPTH_COMPONENT = GLES20.GL_DEPTH_COMPONENT;
	public final int GL_DEPTH_COMPONENT16 = GLES20.GL_DEPTH_COMPONENT16;
	public final int GL_DEPTH_FUNC = GLES20.GL_DEPTH_FUNC;
	public final int GL_DEPTH_RANGE = GLES20.GL_DEPTH_RANGE;
	public final int GL_DEPTH_TEST = GLES20.GL_DEPTH_TEST;
	public final int GL_DEPTH_WRITEMASK = GLES20.GL_DEPTH_WRITEMASK;
	public final int GL_DITHER = GLES20.GL_DITHER;
	public final int GL_DONT_CARE = GLES20.GL_DONT_CARE;
	public final int GL_DST_ALPHA = GLES20.GL_DST_ALPHA;
	public final int GL_DST_COLOR = GLES20.GL_DST_COLOR;
	public final int GL_DYNAMIC_DRAW = GLES20.GL_DYNAMIC_DRAW;
	public final int GL_ELEMENT_ARRAY_BUFFER = GLES20.GL_ELEMENT_ARRAY_BUFFER;
	public final int GL_ELEMENT_ARRAY_BUFFER_BINDING = GLES20.GL_ELEMENT_ARRAY_BUFFER_BINDING;
	public final int GL_EQUAL = GLES20.GL_EQUAL;
	public final int GL_EXTENSIONS = GLES20.GL_EXTENSIONS;
	public final int GL_FALSE = GLES20.GL_FALSE;
	public final int GL_FASTEST = GLES20.GL_FASTEST;
	public final int GL_FIXED = GLES20.GL_FIXED;
	public final int GL_FLOAT = GLES20.GL_FLOAT;
	public final int GL_FLOAT_MAT2 = GLES20.GL_FLOAT_MAT2;
	public final int GL_FLOAT_MAT3 = GLES20.GL_FLOAT_MAT3;
	public final int GL_FLOAT_MAT4 = GLES20.GL_FLOAT_MAT4;
	public final int GL_FLOAT_VEC2 = GLES20.GL_FLOAT_VEC2;
	public final int GL_FLOAT_VEC3 = GLES20.GL_FLOAT_VEC3;
	public final int GL_FLOAT_VEC4 = GLES20.GL_FLOAT_VEC4;
	public final int GL_FRAGMENT_SHADER = GLES20.GL_FRAGMENT_SHADER;
	public final int GL_FRAMEBUFFER = GLES20.GL_FRAMEBUFFER;
	public final int GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = GLES20.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME;
	public final int GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = GLES20.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE;
	public final int GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = GLES20.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE;
	public final int GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = GLES20.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL;
	public final int GL_FRAMEBUFFER_BINDING = GLES20.GL_FRAMEBUFFER_BINDING;
	public final int GL_FRAMEBUFFER_COMPLETE = GLES20.GL_FRAMEBUFFER_COMPLETE;
	public final int GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = GLES20.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT;
	public final int GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS = GLES20.GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS;
	public final int GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = GLES20.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT;
	public final int GL_FRAMEBUFFER_UNSUPPORTED = GLES20.GL_FRAMEBUFFER_UNSUPPORTED;
	public final int GL_FRONT = GLES20.GL_FRONT;
	public final int GL_FRONT_AND_BACK = GLES20.GL_FRONT_AND_BACK;
	public final int GL_FRONT_FACE = GLES20.GL_FRONT_FACE;
	public final int GL_FUNC_ADD = GLES20.GL_FUNC_ADD;
	public final int GL_FUNC_REVERSE_SUBTRACT = GLES20.GL_FUNC_REVERSE_SUBTRACT;
	public final int GL_FUNC_SUBTRACT = GLES20.GL_FUNC_SUBTRACT;
	public final int GL_GENERATE_MIPMAP_HINT = GLES20.GL_GENERATE_MIPMAP_HINT;
	public final int GL_GEQUAL = GLES20.GL_GEQUAL;
	public final int GL_GREATER = GLES20.GL_GREATER;
	public final int GL_GREEN_BITS = GLES20.GL_GREEN_BITS;
	public final int GL_HIGH_FLOAT = GLES20.GL_HIGH_INT;
	public final int GL_HIGH_INT = GLES20.GL_HIGH_INT;
	public final int GL_IMPLEMENTATION_COLOR_READ_FORMAT = GLES20.GL_IMPLEMENTATION_COLOR_READ_FORMAT;
	public final int GL_IMPLEMENTATION_COLOR_READ_TYPE = GLES20.GL_IMPLEMENTATION_COLOR_READ_TYPE;
	public final int GL_INCR = GLES20.GL_INCR;
	public final int GL_INCR_WRAP = GLES20.GL_INCR_WRAP;
	public final int GL_INFO_LOG_LENGTH = GLES20.GL_INFO_LOG_LENGTH;
	public final int GL_INT = GLES20.GL_INT;
	public final int GL_INT_VEC2 = GLES20.GL_INT_VEC2;
	public final int GL_INT_VEC3 = GLES20.GL_INT_VEC3;
	public final int GL_INT_VEC4 = GLES20.GL_INT_VEC4;
	public final int GL_INVALID_ENUM = GLES20.GL_INVALID_ENUM;
	public final int GL_INVALID_FRAMEBUFFER_OPERATION = GLES20.GL_INVALID_FRAMEBUFFER_OPERATION;
	public final int GL_INVALID_OPERATION = GLES20.GL_INVALID_OPERATION;
	public final int GL_INVALID_VALUE = GLES20.GL_INVALID_VALUE;
	public final int GL_INVERT = GLES20.GL_INVERT;
	public final int GL_KEEP = GLES20.GL_KEEP;
	public final int GL_LEQUAL = GLES20.GL_LEQUAL;
	public final int GL_LESS = GLES20.GL_LESS;
	public final int GL_LINEAR = GLES20.GL_LINEAR;
	public final int GL_LINEAR_MIPMAP_LINEAR = GLES20.GL_LINEAR_MIPMAP_LINEAR;
	public final int GL_LINEAR_MIPMAP_NEAREST = GLES20.GL_LINEAR_MIPMAP_NEAREST;
	public final int GL_LINES = GLES20.GL_LINES;
	public final int GL_LINE_LOOP = GLES20.GL_LINE_LOOP;
	public final int GL_LINE_STRIP = GLES20.GL_LINE_STRIP;
	public final int GL_LINE_WIDTH = GLES20.GL_LINE_WIDTH;
	public final int GL_LINK_STATUS = GLES20.GL_LINK_STATUS;
	public final int GL_LOW_FLOAT = GLES20.GL_LOW_FLOAT;
	public final int GL_LOW_INT = GLES20.GL_LOW_INT;
	public final int GL_LUMINANCE = GLES20.GL_LUMINANCE;
	public final int GL_LUMINANCE_ALPHA = GLES20.GL_LUMINANCE_ALPHA;
	public final int GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = GLES20.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS;
	public final int GL_MAX_CUBE_MAP_TEXTURE_SIZE = GLES20.GL_MAX_CUBE_MAP_TEXTURE_SIZE;
	public final int GL_MAX_FRAGMENT_UNIFORM_VECTORS = GLES20.GL_MAX_FRAGMENT_UNIFORM_VECTORS;
	public final int GL_MAX_RENDERBUFFER_SIZE = GLES20.GL_MAX_RENDERBUFFER_SIZE;
	public final int GL_MAX_TEXTURE_IMAGE_UNITS = GLES20.GL_MAX_TEXTURE_IMAGE_UNITS;
	public final int GL_MAX_TEXTURE_SIZE = GLES20.GL_MAX_TEXTURE_SIZE;
	public final int GL_MAX_VARYING_VECTORS = GLES20.GL_MAX_VARYING_VECTORS;
	public final int GL_MAX_VERTEX_ATTRIBS = GLES20.GL_MAX_VERTEX_ATTRIBS;
	public final int GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = GLES20.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS;
	public final int GL_MAX_VERTEX_UNIFORM_VECTORS = GLES20.GL_MAX_VERTEX_UNIFORM_VECTORS;
	public final int GL_MAX_VIEWPORT_DIMS = GLES20.GL_MAX_VIEWPORT_DIMS;
	public final int GL_MEDIUM_FLOAT = GLES20.GL_MEDIUM_FLOAT;
	public final int GL_MEDIUM_INT = GLES20.GL_MEDIUM_INT;
	public final int GL_MIRRORED_REPEAT = GLES20.GL_MIRRORED_REPEAT;
	public final int GL_NEAREST = GLES20.GL_NEAREST;
	public final int GL_NEAREST_MIPMAP_LINEAR = GLES20.GL_NEAREST_MIPMAP_LINEAR;
	public final int GL_NEAREST_MIPMAP_NEAREST = GLES20.GL_NEAREST_MIPMAP_NEAREST;
	public final int GL_NEVER = GLES20.GL_NEVER;
	public final int GL_NICEST = GLES20.GL_NICEST;
	public final int GL_NONE = GLES20.GL_NONE;
	public final int GL_NOTEQUAL = GLES20.GL_NOTEQUAL;
	public final int GL_NO_ERROR = GLES20.GL_NO_ERROR;
	public final int GL_NUM_COMPRESSED_TEXTURE_FORMATS = GLES20.GL_NUM_COMPRESSED_TEXTURE_FORMATS;
	public final int GL_NUM_SHADER_BINARY_FORMATS = GLES20.GL_NUM_SHADER_BINARY_FORMATS;
	public final int GL_ONE = GLES20.GL_ONE;
	public final int GL_ONE_MINUS_CONSTANT_ALPHA = GLES20.GL_ONE_MINUS_CONSTANT_ALPHA;
	public final int GL_ONE_MINUS_CONSTANT_COLOR = GLES20.GL_ONE_MINUS_CONSTANT_COLOR;
	public final int GL_ONE_MINUS_DST_ALPHA = GLES20.GL_ONE_MINUS_DST_ALPHA;
	public final int GL_ONE_MINUS_DST_COLOR = GLES20.GL_ONE_MINUS_DST_COLOR;
	public final int GL_ONE_MINUS_SRC_ALPHA = GLES20.GL_ONE_MINUS_SRC_ALPHA;
	public final int GL_ONE_MINUS_SRC_COLOR = GLES20.GL_ONE_MINUS_SRC_COLOR;
	public final int GL_OUT_OF_MEMORY = GLES20.GL_OUT_OF_MEMORY;
	public final int GL_PACK_ALIGNMENT = GLES20.GL_PACK_ALIGNMENT;
	public final int GL_POINTS = GLES20.GL_POINTS;
	public final int GL_POLYGON_OFFSET_FACTOR = GLES20.GL_POLYGON_OFFSET_FACTOR;
	public final int GL_POLYGON_OFFSET_FILL = GLES20.GL_POLYGON_OFFSET_FILL;
	public final int GL_POLYGON_OFFSET_UNITS = GLES20.GL_POLYGON_OFFSET_UNITS;
	public final int GL_RED_BITS = GLES20.GL_RED_BITS;
	public final int GL_RENDERBUFFER = GLES20.GL_RENDERBUFFER;
	public final int GL_RENDERBUFFER_ALPHA_SIZE = GLES20.GL_RENDERBUFFER_ALPHA_SIZE;
	public final int GL_RENDERBUFFER_BINDING = GLES20.GL_RENDERBUFFER_BINDING;
	public final int GL_RENDERBUFFER_BLUE_SIZE = GLES20.GL_RENDERBUFFER_BLUE_SIZE;
	public final int GL_RENDERBUFFER_DEPTH_SIZE = GLES20.GL_RENDERBUFFER_DEPTH_SIZE;
	public final int GL_RENDERBUFFER_GREEN_SIZE = GLES20.GL_RENDERBUFFER_GREEN_SIZE;
	public final int GL_RENDERBUFFER_HEIGHT = GLES20.GL_RENDERBUFFER_HEIGHT;
	public final int GL_RENDERBUFFER_INTERNAL_FORMAT = GLES20.GL_RENDERBUFFER_INTERNAL_FORMAT;
	public final int GL_RENDERBUFFER_RED_SIZE = GLES20.GL_RENDERBUFFER_RED_SIZE;
	public final int GL_RENDERBUFFER_STENCIL_SIZE = GLES20.GL_RENDERBUFFER_STENCIL_SIZE;
	public final int GL_RENDERBUFFER_WIDTH = GLES20.GL_RENDERBUFFER_WIDTH;
	public final int GL_RENDERER = GLES20.GL_RENDERER;
	public final int GL_REPEAT = GLES20.GL_REPEAT;
	public final int GL_REPLACE = GLES20.GL_REPLACE;
	public final int GL_RGB = GLES20.GL_RGB;
	public final int GL_RGB565 = GLES20.GL_RGB565;
	public final int GL_RGB5_A1 = GLES20.GL_RGB5_A1;
	public final int GL_RGBA = GLES20.GL_RGBA;
	public final int GL_RGBA4 = GLES20.GL_RGBA4;
	public final int GL_SAMPLER_2D = GLES20.GL_SAMPLER_2D;
	public final int GL_SAMPLER_CUBE = GLES20.GL_SAMPLER_CUBE;
	public final int GL_SAMPLES = GLES20.GL_SAMPLES;
	public final int GL_SAMPLE_ALPHA_TO_COVERAGE = GLES20.GL_SAMPLE_ALPHA_TO_COVERAGE;
	public final int GL_SAMPLE_BUFFERS = GLES20.GL_SAMPLE_BUFFERS;
	public final int GL_SAMPLE_COVERAGE = GLES20.GL_SAMPLE_COVERAGE;
	public final int GL_SAMPLE_COVERAGE_INVERT = GLES20.GL_SAMPLE_COVERAGE_INVERT;
	public final int GL_SAMPLE_COVERAGE_VALUE = GLES20.GL_SAMPLE_COVERAGE_VALUE;
	public final int GL_SCISSOR_BOX = GLES20.GL_SCISSOR_BOX;
	public final int GL_SCISSOR_TEST = GLES20.GL_SCISSOR_TEST;
	public final int GL_SHADER_BINARY_FORMATS = GLES20.GL_SHADER_BINARY_FORMATS;
	public final int GL_SHADER_COMPILER = GLES20.GL_SHADER_COMPILER;
	public final int GL_SHADER_SOURCE_LENGTH = GLES20.GL_SHADER_SOURCE_LENGTH;
	public final int GL_SHADER_TYPE = GLES20.GL_SHADER_TYPE;
	public final int GL_SHADING_LANGUAGE_VERSION = GLES20.GL_SHADING_LANGUAGE_VERSION;
	public final int GL_SHORT = GLES20.GL_SHORT;
	public final int GL_SRC_ALPHA = GLES20.GL_SRC_ALPHA;
	public final int GL_SRC_ALPHA_SATURATE = GLES20.GL_SRC_ALPHA_SATURATE;
	public final int GL_SRC_COLOR = GLES20.GL_SRC_COLOR;
	public final int GL_STATIC_DRAW = GLES20.GL_STATIC_DRAW;
	public final int GL_STENCIL_ATTACHMENT = GLES20.GL_STENCIL_ATTACHMENT;
	public final int GL_STENCIL_BACK_FAIL = GLES20.GL_STENCIL_BACK_FAIL;
	public final int GL_STENCIL_BACK_FUNC = GLES20.GL_STENCIL_BACK_FUNC;
	public final int GL_STENCIL_BACK_PASS_DEPTH_FAIL = GLES20.GL_STENCIL_BACK_PASS_DEPTH_FAIL;
	public final int GL_STENCIL_BACK_PASS_DEPTH_PASS = GLES20.GL_STENCIL_BACK_PASS_DEPTH_PASS;
	public final int GL_STENCIL_BACK_REF = GLES20.GL_STENCIL_BACK_REF;
	public final int GL_STENCIL_BACK_VALUE_MASK = GLES20.GL_STENCIL_BACK_VALUE_MASK;
	public final int GL_STENCIL_BACK_WRITEMASK = GLES20.GL_STENCIL_BACK_WRITEMASK;
	public final int GL_STENCIL_BITS = GLES20.GL_STENCIL_BITS;
	public final int GL_STENCIL_BUFFER_BIT = GLES20.GL_STENCIL_BUFFER_BIT;
	public final int GL_STENCIL_CLEAR_VALUE = GLES20.GL_STENCIL_CLEAR_VALUE;
	public final int GL_STENCIL_FAIL = GLES20.GL_STENCIL_FAIL;
	public final int GL_STENCIL_FUNC = GLES20.GL_STENCIL_FUNC;
	public final int GL_STENCIL_INDEX = GLES20.GL_STENCIL_INDEX;
	public final int GL_STENCIL_INDEX8 = GLES20.GL_STENCIL_INDEX8;
	public final int GL_STENCIL_PASS_DEPTH_FAIL = GLES20.GL_STENCIL_PASS_DEPTH_FAIL;
	public final int GL_STENCIL_PASS_DEPTH_PASS = GLES20.GL_STENCIL_PASS_DEPTH_PASS;
	public final int GL_STENCIL_REF = GLES20.GL_STENCIL_REF;
	public final int GL_STENCIL_TEST = GLES20.GL_STENCIL_TEST;
	public final int GL_STENCIL_VALUE_MASK = GLES20.GL_STENCIL_VALUE_MASK;
	public final int GL_STENCIL_WRITEMASK = GLES20.GL_STENCIL_WRITEMASK;
	public final int GL_STREAM_DRAW = GLES20.GL_STREAM_DRAW;
	public final int GL_SUBPIXEL_BITS = GLES20.GL_SUBPIXEL_BITS;
	public final int GL_TEXTURE = GLES20.GL_TEXTURE;
	public final int GL_TEXTURE0 = GLES20.GL_TEXTURE0;
	public final int GL_TEXTURE1 = GLES20.GL_TEXTURE1;
	public final int GL_TEXTURE10 = GLES20.GL_TEXTURE10;
	public final int GL_TEXTURE11 = GLES20.GL_TEXTURE11;
	public final int GL_TEXTURE12 = GLES20.GL_TEXTURE12;
	public final int GL_TEXTURE13 = GLES20.GL_TEXTURE13;
	public final int GL_TEXTURE14 = GLES20.GL_TEXTURE14;
	public final int GL_TEXTURE15 = GLES20.GL_TEXTURE15;
	public final int GL_TEXTURE16 = GLES20.GL_TEXTURE16;
	public final int GL_TEXTURE17 = GLES20.GL_TEXTURE17;
	public final int GL_TEXTURE18 = GLES20.GL_TEXTURE18;
	public final int GL_TEXTURE19 = GLES20.GL_TEXTURE19;
	public final int GL_TEXTURE2 = GLES20.GL_TEXTURE2;
	public final int GL_TEXTURE20 = GLES20.GL_TEXTURE20;
	public final int GL_TEXTURE21 = GLES20.GL_TEXTURE21;
	public final int GL_TEXTURE22 = GLES20.GL_TEXTURE22;
	public final int GL_TEXTURE23 = GLES20.GL_TEXTURE23;
	public final int GL_TEXTURE24 = GLES20.GL_TEXTURE24;
	public final int GL_TEXTURE25 = GLES20.GL_TEXTURE25;
	public final int GL_TEXTURE26 = GLES20.GL_TEXTURE26;
	public final int GL_TEXTURE27 = GLES20.GL_TEXTURE27;
	public final int GL_TEXTURE28 = GLES20.GL_TEXTURE28;
	public final int GL_TEXTURE29 = GLES20.GL_TEXTURE29;
	public final int GL_TEXTURE3 = GLES20.GL_TEXTURE3;
	public final int GL_TEXTURE30 = GLES20.GL_TEXTURE30;
	public final int GL_TEXTURE31 = GLES20.GL_TEXTURE31;
	public final int GL_TEXTURE4 = GLES20.GL_TEXTURE4;
	public final int GL_TEXTURE5 = GLES20.GL_TEXTURE5;
	public final int GL_TEXTURE6 = GLES20.GL_TEXTURE6;
	public final int GL_TEXTURE7 = GLES20.GL_TEXTURE7;
	public final int GL_TEXTURE8 = GLES20.GL_TEXTURE8;
	public final int GL_TEXTURE9 = GLES20.GL_TEXTURE9;
	public final int GL_TEXTURE_2D = GLES20.GL_TEXTURE_2D;
	public final int GL_TEXTURE_BINDING_2D = GLES20.GL_TEXTURE_BINDING_2D;
	public final int GL_TEXTURE_BINDING_CUBE_MAP = GLES20.GL_TEXTURE_BINDING_CUBE_MAP;
	public final int GL_TEXTURE_CUBE_MAP = GLES20.GL_TEXTURE_CUBE_MAP;
	public final int GL_TEXTURE_CUBE_MAP_NEGATIVE_X = GLES20.GL_TEXTURE_CUBE_MAP_NEGATIVE_X;
	public final int GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = GLES20.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y;
	public final int GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = GLES20.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z;
	public final int GL_TEXTURE_CUBE_MAP_POSITIVE_X = GLES20.GL_TEXTURE_CUBE_MAP_POSITIVE_X;
	public final int GL_TEXTURE_CUBE_MAP_POSITIVE_Y = GLES20.GL_TEXTURE_CUBE_MAP_POSITIVE_Y;
	public final int GL_TEXTURE_CUBE_MAP_POSITIVE_Z = GLES20.GL_TEXTURE_CUBE_MAP_POSITIVE_Z;
	public final int GL_TEXTURE_MAG_FILTER = GLES20.GL_TEXTURE_MAG_FILTER;
	public final int GL_TEXTURE_MIN_FILTER = GLES20.GL_TEXTURE_MIN_FILTER;
	public final int GL_TEXTURE_WRAP_S = GLES20.GL_TEXTURE_WRAP_S;
	public final int GL_TEXTURE_WRAP_T = GLES20.GL_TEXTURE_WRAP_T;
	public final int GL_TRIANGLES = GLES20.GL_TRIANGLES;
	public final int GL_TRIANGLE_FAN = GLES20.GL_TRIANGLE_FAN;
	public final int GL_TRIANGLE_STRIP = GLES20.GL_TRIANGLE_STRIP;
	public final int GL_TRUE = GLES20.GL_TRUE;
	public final int GL_UNPACK_ALIGNMENT = GLES20.GL_UNPACK_ALIGNMENT;
	public final int GL_UNSIGNED_BYTE = GLES20.GL_UNSIGNED_BYTE;
	public final int GL_UNSIGNED_SHORT = GLES20.GL_UNSIGNED_SHORT;
	public final int GL_UNSIGNED_SHORT_4_4_4_4 = GLES20.GL_UNSIGNED_SHORT_4_4_4_4;
	public final int GL_UNSIGNED_SHORT_5_5_5_1 = GLES20.GL_UNSIGNED_SHORT_5_5_5_1;
	public final int GL_UNSIGNED_SHORT_5_6_5 = GLES20.GL_UNSIGNED_SHORT_5_6_5;
	public final int GL_VALIDATE_STATUS = GLES20.GL_VALIDATE_STATUS;
	public final int GL_VENDOR = GLES20.GL_VENDOR;
	public final int GL_VERSION = GLES20.GL_VERSION;
	public final int GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = GLES20.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING;
	public final int GL_VERTEX_ATTRIB_ARRAY_ENABLED = GLES20.GL_VERTEX_ATTRIB_ARRAY_ENABLED;
	public final int GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = GLES20.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED;
	public final int GL_VERTEX_ATTRIB_ARRAY_POINTER = GLES20.GL_VERTEX_ATTRIB_ARRAY_POINTER;
	public final int GL_VERTEX_ATTRIB_ARRAY_SIZE = GLES20.GL_VERTEX_ATTRIB_ARRAY_SIZE;
	public final int GL_VERTEX_ATTRIB_ARRAY_STRIDE = GLES20.GL_VERTEX_ATTRIB_ARRAY_STRIDE;
	public final int GL_VERTEX_ATTRIB_ARRAY_TYPE = GLES20.GL_VERTEX_ATTRIB_ARRAY_TYPE;
	public final int GL_VERTEX_SHADER = GLES20.GL_VERTEX_SHADER;
	public final int GL_VIEWPORT = GLES20.GL_VIEWPORT;
	public final int GL_ZERO = GLES20.GL_ZERO;

	// GLES20 METHODS

	public void glActiveTexture(int texture)
	{
		GLES20.glActiveTexture(texture);

	}

	public void glAttachShader(int program, int shader)
	{
		GLES20.glAttachShader(program, shader);
	}

	public void glBindAttribLocation(int program, int index, String name)
	{
		GLES20.glBindAttribLocation(program, index, name);
	}

	public void glBindBuffer(int target, int buffer)
	{
		GLES20.glBindBuffer(target, buffer);
	}

	public void glBindFramebuffer(int target, int framebuffer)
	{
		GLES20.glBindFramebuffer(target, framebuffer);
	}

	public void glBindRenderbuffer(int target, int renderbuffer)
	{
		GLES20.glBindRenderbuffer(target, renderbuffer);
	}

	public void glBindTexture(int target, int texture)
	{
		GLES20.glBindTexture(target, texture);
	}

	public void glBlendColor(float red, float green, float blue, float alpha)
	{
		GLES20.glBlendColor(red, green, blue, alpha);
	}

	public void glBlendEquation(int mode)
	{
		GLES20.glBlendEquation(mode);
	}

	public void glBlendEquationSeparate(int modeRGB, int modeAlpha)
	{
		GLES20.glBlendEquationSeparate(modeRGB, modeAlpha);
	}

	public void glBlendFunc(int sfactor, int dfactor)
	{
		GLES20.glBlendFunc(sfactor, dfactor);
	}

	public void glBlendFuncSeparate(int srcRGB, int dstRGB, int srcAlpha, int dstAlpha)
	{
		GLES20.glBlendFuncSeparate(srcRGB, dstRGB, srcAlpha, dstAlpha);
	}

	//public void  glBufferData(int target, int size, Buffer data, int usage) 
	public void glBufferDataI(int target, int size, int[] data, int usage)
	{
		ByteBuffer pbb = ByteBuffer.allocateDirect(data.length * 4);
		pbb.order(ByteOrder.nativeOrder());
		IntBuffer pib = pbb.asIntBuffer();
		pib.put(data);
		pib.position(0);
		GLES20.glBufferData(target, size, pib, usage);
	}

	public void glBufferDataF(int target, int size, float[] data, int usage)
	{
		ByteBuffer pbb = ByteBuffer.allocateDirect(data.length * 4);
		pbb.order(ByteOrder.nativeOrder());
		FloatBuffer pfb = pbb.asFloatBuffer();
		pfb.put(data);
		pfb.position(0);
		GLES20.glBufferData(target, size, pfb, usage);
	}

	//public void  glBufferSubData(int target, int offset, int size, Buffer data)	
	public void glBufferSubDataI(int target, int offset, int size, int[] data)
	{
		ByteBuffer pbb = ByteBuffer.allocateDirect(data.length * 4);
		pbb.order(ByteOrder.nativeOrder());
		IntBuffer pib = pbb.asIntBuffer();
		pib.put(data);
		pib.position(0);
		GLES20.glBufferSubData(target, offset, size, pib);
	}
	
	public void glBufferData(int target, int size, Buffer buffer, int type) {
		GLES20.glBufferData(target, size, buffer, type);
	}

	public void glBufferSubDataF(int target, int offset, int size, float[] data)
	{
		ByteBuffer pbb = ByteBuffer.allocateDirect(data.length * 4);
		pbb.order(ByteOrder.nativeOrder());
		FloatBuffer pfb = pbb.asFloatBuffer();
		pfb.put(data);
		pfb.position(0);
		GLES20.glBufferSubData(target, offset, size, pfb);
	}

	public int glCheckFramebufferStatus(int target)
	{
		return GLES20.glCheckFramebufferStatus(target);
	}

	public void glClear(int mask)
	{
		GLES20.glClear(mask);
	}

	public void glClearColor(float red, float green, float blue, float alpha)
	{
		GLES20.glClearColor(red, green, blue, alpha);
	}

	public void glClearDepthf(float depth)
	{
		GLES20.glClearDepthf(depth);
	}

	public void glClearStencil(int s)
	{
		GLES20.glClearStencil(s);
	}

	public void glColorMask(boolean red, boolean green, boolean blue, boolean alpha)
	{
		GLES20.glColorMask(red, green, blue, alpha);
	}

	public void glCompileShader(int shader)
	{
		GLES20.glCompileShader(shader);
	}

	//public void  glCompressedTexImage2D(int target, int level, int internalformat, int width, int height, int border, int imageSize, Buffer data)
	public void glCompressedTexImage2D(int target, int level, int internalformat, int width, int height, int border, int imageSize,
			Buffer data)
	{
		GLES20.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data);
	}

	//public void  glCompressedTexSubImage2D(int target, int level, int xoffset, int yoffset, int width, int height, int format, int imageSize, Buffer data)  
	public void glCompressedTexSubImage2D(int target, int level, int xoffset, int yoffset, int width, int height, int format,
			int imageSize, Buffer data)
	{
		GLES20.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data);
	}

	public void glCopyTexImage2D(int target, int level, int internalformat, int x, int y, int width, int height, int border)
	{
		GLES20.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border);
	}

	public void glCopyTexSubImage2D(int target, int level, int xoffset, int yoffset, int x, int y, int width, int height)
	{
		GLES20.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height);
	}

	public int glCreateProgram()
	{
		return GLES20.glCreateProgram();
	}

	public int glCreateShader(int type)
	{
		return GLES20.glCreateShader(type);
	}

	public void glCullFace(int mode)
	{
		GLES20.glCullFace(mode);
	}

	public void glDeleteBuffers(int n, int[] buffers, int offset)
	{
		GLES20.glDeleteBuffers(n, buffers, offset);
	}

	//public void  glDeleteBuffers(int n, IntBuffer buffers)

	public void glDeleteFramebuffers(int n, int[] framebuffers, int offset)
	{
		GLES20.glDeleteFramebuffers(n, framebuffers, offset);
	}

	//public void  glDeleteFramebuffers(int n, IntBuffer framebuffers)

	public void glDeleteProgram(int program)
	{
		GLES20.glDeleteProgram(program);
	}

	//public void  glDeleteRenderbuffers(int n, IntBuffer renderbuffers)  

	public void glDeleteRenderbuffers(int n, int[] renderbuffers, int offset)
	{
		GLES20.glDeleteRenderbuffers(n, renderbuffers, offset);
	}

	public void glDeleteShader(int shader)
	{
		GLES20.glDeleteShader(shader);
	}

	public void glDeleteTextures(int n, int[] textures, int offset)
	{
		GLES20.glDeleteTextures(n, textures, offset);
	}

	//public void  glDeleteTextures(int n, IntBuffer textures)

	public void glDepthFunc(int func)
	{
		GLES20.glDepthFunc(func);
	}

	public void glDepthMask(boolean flag)
	{
		GLES20.glDepthMask(flag);
	}

	public void glDepthRangef(float zNear, float zFar)
	{
		GLES20.glDepthRangef(zNear, zFar);
	}

	public void glDetachShader(int program, int shader)
	{
		GLES20.glDetachShader(program, shader);
	}

	public void glDisable(int cap)
	{
		GLES20.glDisable(cap);
	}

	public void glDisableVertexAttribArray(int index)
	{
		GLES20.glDisableVertexAttribArray(index);
	}

	public void glDrawArrays(int mode, int first, int count)
	{
		GLES20.glDrawArrays(mode, first, count);
	}

	//public void  glDrawElements(int mode, int count, int type, Buffer indices)
	public void glDrawElements(int mode, int count, short[] indices)
	{
		ByteBuffer ibb = ByteBuffer.allocateDirect(indices.length * 2);
		ibb.order(ByteOrder.nativeOrder());
		ShortBuffer isb = ibb.asShortBuffer();
		isb.put(indices);
		isb.position(0);
		GLES20.glDrawElements(mode, count, GL_UNSIGNED_SHORT, isb);
		
		// GLES20.glDrawElements(GLES20.GL_TRIANGLE_FAN, 4, GLES20.GL_UNSIGNED_SHORT, i * 4 * 2);
	}
	
	public void glDrawElements2(int mode, int count, int type, int offset) {
		GLES20.glDrawElements(mode, count, type, offset);
	}

	public void glEnable(int cap)
	{
		GLES20.glEnable(cap);
	}

	public void glEnableVertexAttribArray(int index)
	{
		GLES20.glEnableVertexAttribArray(index);
	}

	public void glFinish()
	{
		GLES20.glFinish();
	}

	public void glFlush()
	{
		GLES20.glFlush();
	}

	public void glFramebufferRenderbuffer(int target, int attachment, int renderbuffertarget, int renderbuffer)
	{
		GLES20.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer);
	}

	public void glFramebufferTexture2D(int target, int attachment, int textarget, int texture, int level)
	{
		GLES20.glFramebufferTexture2D(target, attachment, textarget, texture, level);
	}

	public void glFrontFace(int mode)
	{
		GLES20.glFrontFace(mode);
	}

	//public void  glGenBuffers(int n, IntBuffer buffers)
	
	public void glGenBuffers(int n, int[] buffers, int offset)
	{
		GLES20.glGenBuffers(n, buffers, offset);
	}

	//public void  glGenFramebuffers(int n, IntBuffer framebuffers)

	public void glGenFramebuffers(int n, int[] framebuffers, int offset)
	{
		GLES20.glGenFramebuffers(n, framebuffers, offset);
	}

	//public void  glGenRenderbuffers(int n, IntBuffer renderbuffers)

	public void glGenRenderbuffers(int n, int[] renderbuffers, int offset)
	{
		GLES20.glGenRenderbuffers(n, renderbuffers, offset);
	}

	//public void  glGenTextures(int n, IntBuffer textures)

	public void glGenTextures(int n, int[] textures, int offset)
	{
		GLES20.glGenTextures(n, textures, offset);
	}

	public void glGenerateMipmap(int target)
	{
		GLES20.glGenerateMipmap(target);
	}

	//public void  glGetActiveAttrib(int program, int index, int bufsize, IntBuffer length, IntBuffer size, IntBuffer type, byte name)  

	public void glGetActiveAttrib(int program, int index, int bufsize, int[] length, int lengthOffset, int[] size, int sizeOffset,
			int[] type, int typeOffset, byte[] name, int nameOffset)
	{
		GLES20.glGetActiveAttrib(program, index, bufsize, length, lengthOffset, size, sizeOffset, type, typeOffset, name,
				nameOffset);
	}

	//public void  glGetActiveUniform(int program, int index, int bufsize, IntBuffer length, IntBuffer size, IntBuffer type, byte name)

	public void glGetActiveUniform(int program, int index, int bufsize, int[] length, int lengthOffset, int[] size, int sizeOffset,
			int[] type, int typeOffset, byte[] name, int nameOffset)
	{
		GLES20.glGetActiveUniform(program, index, bufsize, length, lengthOffset, size, sizeOffset, type, typeOffset, name,
				nameOffset);
	}

	public void glGetAttachedShaders(int program, int maxcount, int[] count, int countOffset, int[] shaders, int shadersOffset)
	{
		GLES20.glGetAttachedShaders(program, maxcount, count, countOffset, shaders, shadersOffset);
	}

	//public void  glGetAttachedShaders(int program, int maxcount, IntBuffer count, IntBuffer shaders)  

	public int glGetAttribLocation(int program, String name)
	{
		return GLES20.glGetAttribLocation(program, name);
	}

	public void glGetBooleanv(int pname, boolean[] params, int offset)
	{
		GLES20.glGetBooleanv(pname, params, offset);
	}

	//public void  glGetBooleanv(int pname, IntBuffer params) 

	//public void  glGetBufferParameteriv(int target, int pname, IntBuffer params) 

	public void glGetBufferParameteriv(int target, int pname, int[] params, int offset)
	{
		GLES20.glGetBufferParameteriv(target, pname, params, offset);
	}

	public int glGetError()
	{
		return GLES20.glGetError();
	}

	//public void  glGetFloatv(int pname, FloatBuffer params)

	public void glGetFloatv(int pname, float[] params, int offset)
	{
		GLES20.glGetFloatv(pname, params, offset);
	}

	//public void  glGetFramebufferAttachmentParameteriv(int target, int attachment, int pname, IntBuffer params)  

	public void glGetFramebufferAttachmentParameteriv(int target, int attachment, int pname, int[] params, int offset)
	{
		GLES20.glGetFramebufferAttachmentParameteriv(target, attachment, pname, params, offset);
	}

	//public void  glGetIntegerv(int pname, IntBuffer params) 

	public void glGetIntegerv(int pname, int[] params, int offset)
	{
		GLES20.glGetIntegerv(pname, params, offset);
	}

	public String glGetProgramInfoLog(int program)
	{
		return GLES20.glGetProgramInfoLog(program);
	}

	//public void  glGetProgramiv(int program, int pname, IntBuffer params)

	public void glGetProgramiv(int program, int pname, int[] params, int offset)
	{
		GLES20.glGetProgramiv(program, pname, params, offset);
	}

	public void glGetRenderbufferParameteriv(int target, int pname, int[] params, int offset)
	{
		GLES20.glGetRenderbufferParameteriv(target, pname, params, offset);
	}

	//public void  glGetRenderbufferParameteriv(int target, int pname, IntBuffer params) 

	public String glGetShaderInfoLog(int shader)
	{
		return GLES20.glGetShaderInfoLog(shader);
	}

	//public void  glGetShaderPrecisionFormat(int shadertype, int precisiontype, IntBuffer range, IntBuffer precision)  

	public void glGetShaderPrecisionFormat(int shadertype, int precisiontype, int[] range, int rangeOffset, int[] precision,
			int precisionOffset)
	{
		GLES20.glGetShaderPrecisionFormat(shadertype, precisiontype, range, rangeOffset, precision, precisionOffset);
	}

	public void glGetShaderSource(int shader, int bufsize, int[] length, int lengthOffset, byte[] source, int sourceOffset)
	{
		GLES20.glGetShaderSource(shader, bufsize, length, lengthOffset, source, sourceOffset);

	}

	//public void  glGetShaderSource(int shader, int bufsize, IntBuffer length, byte source)  

	//public void  glGetShaderiv(int shader, int pname, IntBuffer params) 

	public void glGetShaderiv(int shader, int pname, int[] params, int offset)
	{
		GLES20.glGetShaderiv(shader, pname, params, offset);
	}

	public String glGetString(int name)
	{
		return GLES20.glGetString(name);
	}

	public void glGetTexParameterfv(int target, int pname, float[] params, int offset)
	{
		GLES20.glGetTexParameterfv(target, pname, params, offset);
	}

	//public void  glGetTexParameterfv(int target, int pname, FloatBuffer params)

	public void glGetTexParameteriv(int target, int pname, int[] params, int offset)
	{
		GLES20.glGetTexParameteriv(target, pname, params, offset);
	}

	//public void  glGetTexParameteriv(int target, int pname, IntBuffer params)

	public int glGetUniformLocation(int program, String name)
	{
		return GLES20.glGetUniformLocation(program, name);
	}

	//public void  glGetUniformfv(int program, int location, FloatBuffer params) 

	public void glGetUniformfv(int program, int location, float[] params, int offset)
	{
		GLES20.glGetUniformfv(program, location, params, offset);
	}

	//public void  glGetUniformiv(int program, int location, IntBuffer params)

	public void glGetUniformiv(int program, int location, int[] params, int offset)
	{
		GLES20.glGetUniformiv(program, location, params, offset);
	}

	public void glGetVertexAttribfv(int index, int pname, float[] params, int offset)
	{
		GLES20.glGetVertexAttribfv(index, pname, params, offset);
	}

	//public void  glGetVertexAttribfv(int index, int pname, FloatBuffer params)  

	public void glGetVertexAttribiv(int index, int pname, int[] params, int offset)
	{
		GLES20.glGetVertexAttribiv(index, pname, params, offset);
	}

	//public void  glGetVertexAttribiv(int index, int pname, IntBuffer params) 

	public void glHint(int target, int mode)
	{
		GLES20.glHint(target, mode);
	}

	public boolean glIsBuffer(int buffer)
	{
		return GLES20.glIsBuffer(buffer);
	}

	public boolean glIsEnabled(int cap)
	{
		return GLES20.glIsEnabled(cap);
	}

	public boolean glIsFramebuffer(int framebuffer)
	{
		return GLES20.glIsFramebuffer(framebuffer);
	}

	public boolean glIsProgram(int program)
	{
		return GLES20.glIsProgram(program);
	}

	public boolean glIsRenderbuffer(int renderbuffer)
	{
		return GLES20.glIsRenderbuffer(renderbuffer);
	}

	public boolean glIsShader(int shader)
	{
		return GLES20.glIsShader(shader);
	}

	public boolean glIsTexture(int texture)
	{
		return GLES20.glIsTexture(texture);
	}

	public void glLineWidth(float width)
	{
		GLES20.glLineWidth(width);
	}

	public void glLinkProgram(int program)
	{
		GLES20.glLinkProgram(program);
	}

	public void glPixelStorei(int pname, int param)
	{
		GLES20.glPixelStorei(pname, param);
	}

	public void glPolygonOffset(float factor, float units)
	{
		GLES20.glPolygonOffset(factor, units);
	}

	//public void  glReadPixels(int x, int y, int width, int height, int format, int type, Buffer pixels)
	/**
	 *Returns an array containing the ARGB pixels in the specified area of the frame-buffer.
	 */
	public int[] glReadPixels(int x, int y, int width, int height)
	{
		int b[] = new int[width * height];
		int bt[] = new int[width * height];
		IntBuffer ib = IntBuffer.wrap(b);
		ib.position(0);
		GLES20.glReadPixels(x, y, width, height, GL10.GL_RGBA, GL10.GL_UNSIGNED_BYTE, ib);
		for (int i = 0; i < height; i++)
		{//remember, that OpenGL bitmap is incompatible with Android bitmap
			//and so, some correction need.        
			for (int j = 0; j < width; j++)
			{
				int pix = b[i * width + j];
				int pb = (pix >> 16) & 0xff;
				int pr = (pix << 16) & 0x00ff0000;
				int pix1 = (pix & 0xff00ff00) | pr | pb;
				bt[(height - i - 1) * width + j] = pix1;
			}
		}
		return bt;
	}

	/**
	 *Returns the specified portion of the framebuffer as an ARGB_8888 Bitmap);
	 */
	public Bitmap glReadPixelBitmap(int x, int y, int width, int height)
	{
		int bt[] = glReadPixels(x, y, width, height);
		Bitmap sb = Bitmap.createBitmap(bt, width, height, Bitmap.Config.ARGB_8888);
		return sb;
	}

	public void glReleaseShaderCompiler()
	{
		GLES20.glReleaseShaderCompiler();
	}

	public void glRenderbufferStorage(int target, int internalformat, int width, int height)
	{
		GLES20.glRenderbufferStorage(target, internalformat, width, height);
	}

	public void glSampleCoverage(float value, boolean invert)
	{
		GLES20.glSampleCoverage(value, invert);
	}

	public void glScissor(int x, int y, int width, int height)
	{
		GLES20.glScissor(x, y, width, height);
	}

	//public void  glShaderBinary(int n, IntBuffer shaders, int binaryformat, Buffer binary, int length)  

	//public void  glShaderBinary(int n, int[] shaders, int offset, int binaryformat, Buffer binary, int length)  

	public void glShaderSource(int shader, String string)
	{
		GLES20.glShaderSource(shader, string);
	}

	public void glStencilFunc(int func, int ref, int mask)
	{
		GLES20.glStencilFunc(func, ref, mask);
	}

	public void glStencilFuncSeparate(int face, int func, int ref, int mask)
	{
		GLES20.glStencilFuncSeparate(face, func, ref, mask);
	}

	public void glStencilMask(int mask)
	{
		GLES20.glStencilMask(mask);
	}

	public void glStencilMaskSeparate(int face, int mask)
	{
		GLES20.glStencilMaskSeparate(face, mask);
	}

	public void glStencilOp(int fail, int zfail, int zpass)
	{
		GLES20.glStencilOp(fail, zfail, zpass);
	}

	public void glStencilOpSeparate(int face, int fail, int zfail, int zpass)
	{
		GLES20.glStencilOpSeparate(face, fail, zfail, zpass);
	}

	//public void  glTexImage2D(int target, int level, int internalformat, int width, int height, int border, int format, int type, Buffer pixels)  
	public void glTexImage2D(int target, int level, Bitmap bitmap, int border)
	{
		GLUtils.texImage2D(target, level, bitmap, border);
	}

	public void glTexParameterf(int target, int pname, float param)
	{
		GLES20.glTexParameterf(target, pname, param);
	}

	public void glTexParameterfv(int target, int pname, float[] params, int offset)
	{
		GLES20.glTexParameterfv(target, pname, params, offset);
	}

	//public void  glTexParameterfv(int target, int pname, FloatBuffer params) 

	public void glTexParameteri(int target, int pname, int param)
	{
		GLES20.glTexParameteri(target, pname, param);
	}

	//public void  glTexParameteriv(int target, int pname, IntBuffer params) 

	public void glTexParameteriv(int target, int pname, int[] params, int offset)
	{
		GLES20.glTexParameteriv(target, pname, params, offset);
	}

	//public void  glTexSubImage2D(int target, int level, int xoffset, int yoffset, int width, int height, int format, int type, Buffer pixels)  
	public void glTexSubImage2D(int target, int level, int xoffset, int yoffset, Bitmap bitmap)
	{
		GLUtils.texSubImage2D(target, level, xoffset, yoffset, bitmap);
	}

	public void glUniform1f(int location, float x)
	{
		GLES20.glUniform1f(location, x);
	}

	public void glUniform1fv(int location, int count, float[] v, int offset)
	{
		GLES20.glUniform1fv(location, count, v, offset);
	}

	//public void  glUniform1fv(int location, int count, FloatBuffer v) 

	public void glUniform1i(int location, int x)
	{
		GLES20.glUniform1i(location, x);
	}

	//public void  glUniform1iv(int location, int count, IntBuffer v) 

	public void glUniform1iv(int location, int count, int[] v, int offset)
	{
		GLES20.glUniform1iv(location, count, v, offset);
	}

	public void glUniform2f(int location, float x, float y)
	{
		GLES20.glUniform2f(location, x, y);
	}

	//public void  glUniform2fv(int location, int count, FloatBuffer v) 

	public void glUniform2fv(int location, int count, float[] v, int offset)
	{
		GLES20.glUniform2fv(location, count, v, offset);
	}

	public void glUniform2i(int location, int x, int y)
	{
		GLES20.glUniform2i(location, x, y);
	}

	//public void  glUniform2iv(int location, int count, IntBuffer v)

	public void glUniform2iv(int location, int count, int[] v, int offset)
	{
		GLES20.glUniform2iv(location, count, v, offset);
	}

	public void glUniform3f(int location, float x, float y, float z)
	{
		GLES20.glUniform3f(location, x, y, z);
	}

	public void glUniform3fv(int location, int count, float[] v, int offset)
	{
		GLES20.glUniform3fv(location, count, v, offset);
	}

	//public void  glUniform3fv(int location, int count, FloatBuffer v)

	public void glUniform3i(int location, int x, int y, int z)
	{
		GLES20.glUniform3i(location, x, y, z);
	}

	//public void  glUniform3iv(int location, int count, IntBuffer v)

	public void glUniform3iv(int location, int count, int[] v, int offset)
	{
		GLES20.glUniform3iv(location, count, v, offset);
	}

	public void glUniform4f(int location, float x, float y, float z, float w)
	{
		GLES20.glUniform4f(location, x, y, z, w);
	}

	//public void  glUniform4fv(int location, int count, FloatBuffer v) 

	public void glUniform4fv(int location, int count, float[] v, int offset)
	{
		GLES20.glUniform4fv(location, count, v, offset);
	}

	public void glUniform4i(int location, int x, int y, int z, int w)
	{
		GLES20.glUniform4i(location, x, y, z, w);
	}

	public void glUniform4iv(int location, int count, int[] v, int offset)
	{
		GLES20.glUniform4iv(location, count, v, offset);
	}

	//public void  glUniform4iv(int location, int count, IntBuffer v) 

	//public void  glUniformMatrix2fv(int location, int count, boolean transpose, FloatBuffer value)  

	public void glUniformMatrix2fv(int location, int count, boolean transpose, float[] value, int offset)
	{
		GLES20.glUniformMatrix2fv(location, count, transpose, value, offset);
	}

	public void glUniformMatrix3fv(int location, int count, boolean transpose, float[] value, int offset)
	{
		GLES20.glUniformMatrix3fv(location, count, transpose, value, offset);

	}

	//public void  glUniformMatrix3fv(int location, int count, boolean transpose, FloatBuffer value)  

	public void glUniformMatrix4fv(int location, int count, boolean transpose, float[] value, int offset)
	{
		GLES20.glUniformMatrix4fv(location, count, transpose, value, offset);

	}

	//public void  glUniformMatrix4fv(int location, int count, boolean transpose, FloatBuffer value)  

	public void glUseProgram(int program)
	{
		GLES20.glUseProgram(program);
	}

	public void glValidateProgram(int program)
	{
		GLES20.glValidateProgram(program);
	}

	public void glVertexAttrib1f(int indx, float x)
	{
		GLES20.glVertexAttrib1f(indx, x);
	}

	//public void  glVertexAttrib1fv(int indx, FloatBuffer values)

	public void glVertexAttrib1fv(int indx, float[] values, int offset)
	{
		GLES20.glVertexAttrib1fv(indx, values, offset);
	}

	public void glVertexAttrib2f(int indx, float x, float y)
	{
		GLES20.glVertexAttrib2f(indx, x, y);
	}

	public void glVertexAttrib2fv(int indx, float[] values, int offset)
	{
		GLES20.glVertexAttrib2fv(indx, values, offset);
	}

	//public void  glVertexAttrib2fv(int indx, FloatBuffer values) 

	public void glVertexAttrib3f(int indx, float x, float y, float z)
	{
		GLES20.glVertexAttrib3f(indx, x, y, z);
	}

	//public void  glVertexAttrib3fv(int indx, FloatBuffer values)

	public void glVertexAttrib3fv(int indx, float[] values, int offset)
	{
		GLES20.glVertexAttrib3fv(indx, values, offset);
	}

	public void glVertexAttrib4f(int indx, float x, float y, float z, float w)
	{
		GLES20.glVertexAttrib4f(indx, x, y, z, w);
	}

	//public void  glVertexAttrib4fv(int indx, FloatBuffer values)

	public void glVertexAttrib4fv(int indx, float[] values, int offset)
	{
		GLES20.glVertexAttrib4fv(indx, values, offset);
	}

	public void glVertexAttribPointer(int indx, int size, int type, boolean normalized, int stride, int offset)
	{
		GLES20.glVertexAttribPointer(indx, size, type, normalized, stride, offset);
	}
	
	//public void  glVertexAttribPointer(int indx, int size, int type, boolean normalized, int stride, Buffer ptr) 
	public void glVertexAttribPointerF(int indx, int size, boolean normalized, int stride, float[] ptr)
	{
		// NOTE: glVertexAttribPointer() is spposedly broken for VBOs on Android 2.2.
		// Use the patched GLES20Fix class to call these.
		//https://code.google.com/p/gdc2011-android-opengl/source/browse/trunk/src/com/example/gdc11/GDC11Activity.java
/*		public class GLES20Fix
		{
			native public static void glDrawElements(int mode, int count, int type, int offset);
			native public static void glVertexAttribPointer(int index, int size, int type,
					boolean normalized, int stride, int offset);

			private GLES20Fix()
			{
			

			static
			{
				System.loadLibrary("GLES20Fix");
			}
		}
*/
		ByteBuffer pbb = ByteBuffer.allocateDirect(ptr.length * 4);
		pbb.order(ByteOrder.nativeOrder());
		FloatBuffer pfb = pbb.asFloatBuffer();
		pfb.put(ptr);
		pfb.position(0);
		GLES20.glVertexAttribPointer(indx, size, GL_FLOAT, normalized, stride, pfb);
	}

	public void glViewport(int x, int y, int width, int height)
	{
		GLES20.glViewport(x, y, width, height);
	}

	// GLU Utilities

	/**
	 *Return an error string from a GL or GLU error code.
	 */
	public String gluErrorString(int error)
	{
		return GLU.gluErrorString(error);
	}

	/**
	 *Define a viewing transformation in terms of an eye point, a center of view, and an up vector.
	 *  eyeX  eye point X 
	 *  eyeY  eye point Y
	 *  eyeZ  eye point Z
	 *  centerX  center of view X
	 *  centerY  center of view Y
	 *  centerZ  center of view Z
	 *  upX  up vector X
	 *  upY  up vector Y
	 *  upZ  up vector Z 
	 *  	 */
	public void gluLookAt(float eyeX, float eyeY, float eyeZ, float centerX, float centerY, float centerZ, float upX, float upY,
			float upZ)
	{
		GLU.gluLookAt(gl, eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
	}

	/**
	 *Set up a 2D orthographic projection matrix.
	 */
	public void gluOrtho2D(float left, float right, float bottom, float top)
	{
		GLU.gluOrtho2D(gl, left, right, bottom, top);
	}

	/**
	 *Set up a perspective projection matrix.
	 *fovy  specifies the field of view angle, in degrees, in the Y direction.
	 *aspect  specifies the aspect ratio that determines the field of view in the x direction.
	 *  Aspect ratio is the ratio of x (width) to y (height).
	 *zNear  specifies the distance from the viewer to the near clipping plane (always positive).
	 *zFar  specifies the distance from the viewer to the far clipping plane (always positive).  
	 */
	public void gluPerspective(float fovy, float aspect, float zNear, float zFar)
	{
		GLU.gluPerspective(gl, fovy, aspect, zNear, zFar);
	}

	// End of GL2 Class

	//################################################################

	/**
	 *An implementation of SurfaceView that uses the dedicated surface for displaying OpenGL
	 *rendering. A GLSurfaceView provides the following features:
	 *
	 *  Manages a surface, which is a special piece of memory that can be composited into the Android view system.
	 *  Manages an EGL display, which enables OpenGL to render into a surface.
	 *  Renders on a dedicated thread to decouple rendering performance from the UI thread.
	 *  Supports both on-demand and continuous rendering. 
	 *  
	 *At present a reasonable set of drawing defaults are used to set up the GLSurfaceView.
	 *More control may be available in a future release of this library
	 *glFrustrumf must be called in the SurfaceChanged event to define a frustrum
	 *otherwise the GLSurfaceView is ready to be drawn on.
	 *
	 *Presently there is an unwanted field "ba" visible to Basic4android. Ignore this field.
	 *This field appears due to a feature (bug?) in the Basic4android documentation doclet.
	 *
	 *This view raises the following three events all of which run on a separate rendering thread
	 *and NOT on the main thread of the application. Because of this cosw running in these events
	 *must no attempt to manipulate GUI elements. The RunOnGuiThread method can be used to
	 *invoke a Sub to tun on the main thread and QueueEvent used to invoke a Sub to run on the
	 *rendering thread.
	 *
	 *SurfaceCreated
	 *Called when the surface is created or recreated.
	 *Called when the rendering thread starts and whenever the EGL context is lost.
	 *The EGL context will typically be lost when the Android device awakes after going to sleep.
	 *Since this method is called at the beginning of rendering, as well as every time the EGL
	 *context is lost, this method is a convenient place to put code to create resources that need
	 *to be created when the rendering starts, and that need to be recreated when the EGL context is lost.
	 *Textures are an example of a resource that you might want to create here. 
	 *Note that when the EGL context is lost, all OpenGL resources associated with that context will be
	 *automatically deleted. You do not need to call the corresponding "glDelete" methods such as
	 *glDeleteTextures to manually delete these lost resources. 
	 *
	 *SurfaceChanged
	 *Called when the surface has changed size.
	 *Called after the surface is created and whenever the OpenGL ES surface size changes.
	 *Typically you will set your viewport here. If your camera is fixed then you could also set your
	 *projection matrix here
	 *
	 *Draw
	 *Called to draw the current frame.
	 */
	@ShortName("GLSurfaceView2")
	@Events(values = { "Click", "LongClick", "Draw(gl As GL2) 'The view wants to be drawn using the supplied GL10",
			"SurfaceChanged(gl As GL2, width As Int, height As Int) 'Called when the surface has changed size.",
			"SurfaceCreated(gl As GL2) 'Called when the surface is created or recreated." })
	@ActivityObject
	public static class GLSurfaceViewWrapper2 extends ViewWrapper<GLSurfaceView>
	{

		private int taskId = 0;
		private Exception e;

		/**
		 *Initializes the GLSurfaceView in the specified render mode installing a config chooser
		 *which will choose a config as close to 16-bit RGB as possible, with a depth buffer as
		 *close to 16-bits as possible.
		 *
		 *GLSurfaceView.setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
		 *GLSurfaceView.setEGLConfigChooser(true);
		 *GLSurfaceView.setRenderer(new myRenderer(this, EventName));
		 *setRenderMode(RenderMode);

		 */
		public void Initialize(BA ba, int RenderMode, String EventName)
		{
			super.Initialize(ba, EventName);
			((GLSurfaceView)getObject()).setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
			((GLSurfaceView)getObject()).setEGLConfigChooser(true);
			((GLSurfaceView)getObject()).setRenderer(new myRenderer(this, EventName));
			setRenderMode(RenderMode);
		}
		
		/**
		 *Initializes the GLSurfaceView in the specified render mode installing a config chooser
		 *which will choose a config with 8bit RGBA with the specified depth size and stencil size.
		 *
		 *GLSurfaceView.setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
		 *GLSurfaceView.setEGLConfigChooser(8, 8, 8, 8, depthSize, stencilSize);
		 *GLSurfaceView.getHolder( ).setFormat( PixelFormat.RGBA_8888 );
		 *GLSurfaceView.setRenderer(new myRenderer(this, EventName));
		 *setRenderMode(RenderMode);
		 */
		public void Initialize2(BA ba, int RenderMode, String EventName, int depthSize, int stencilSize)
		{
			super.Initialize(ba, EventName);
			((GLSurfaceView)getObject()).setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
			((GLSurfaceView)getObject()).setEGLConfigChooser(8, 8, 8, 8, depthSize, stencilSize);
			((GLSurfaceView)getObject()).getHolder().setFormat(PixelFormat.RGBA_8888 );
			((GLSurfaceView)getObject()).setRenderer(new myRenderer(this, EventName));
			setRenderMode(RenderMode);
		}
		
		
		/**
		 *Initializes the GLSurfaceView in the specified render mode with all default values
		 *
		 *GLSurfaceView.setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
		 *GLSurfaceView.setRenderer(new myRenderer(this, EventName));
		 *setRenderMode(RenderMode);
		 */
		public void Initialize3(BA ba, int RenderMode, String EventName)
		{
			super.Initialize(ba, EventName);
			((GLSurfaceView)getObject()).setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
			((GLSurfaceView)getObject()).setRenderer(new myRenderer(this, EventName));
			setRenderMode(RenderMode);
		}
		
		private String myEventName;
		/**
		 *Initializes the GLSurfaceView as OpenGL ES 2.0
		 *This is one of a set of Initialisations that let you do a customised Initialisation
		 *This should be called first and is required
		 *
		 *GLSurfaceView.setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
		 */
		public void InitializeGL2(BA ba, String EventName)
		{
			myEventName = EventName;
			super.Initialize(ba, EventName);
			((GLSurfaceView)getObject()).setEGLContextClientVersion(2); // Pick an OpenGL ES 2.0 context.
		}
	
		/**
		 *Initializes the EGLConfigChooser
		 *This is one of a set of Initialisations that let you build a customise Initialisation
		 *This should be called second, if at all
		 *
		 *GLSurfaceView.setEGLConfigChooser(redSize, greenSize, blueSize, alphaSize, depthSize, stencilSize);
		 */
		public void InitializeConfig(int redSize, int greenSize, int blueSize, int alphaSize, int depthSize, int stencilSize)
		{
			((GLSurfaceView)getObject()).setEGLConfigChooser(redSize, greenSize, blueSize, alphaSize, depthSize, stencilSize);
		}
	
		/**
		 *Initializes the EGLConfigChooser
		 *This is one of a set of Initialisations that let you build a customise Initialisation
		 *This should be called third, if at all
		 *
		 *The integer values for PixelFormat can be found here
		 *<link>http://developer.android.com/reference/android/graphics/PixelFormat.html|http://developer.android.com/reference/android/graphics/PixelFormat.html</link>
		 *
		 *GLSurfaceView.getHolder().setFormat(PixelFormat);
		 */
		public void InitializeFormat(int PixelFormat)
		{
			((GLSurfaceView)getObject()).getHolder().setFormat( PixelFormat);
		}

		/**
		 *Initializes the Renderer.
		 *This is one of a set of Initialisations that let you build a customise Initialisation
		 *This should be called last and is required
		 *
		 *GLSurfaceView.setRenderer(new myRenderer(this, EventName));
		 *setRenderMode(RenderMode);
		 */
		public void InitializeRender(int RenderMode)
		{
			((GLSurfaceView)getObject()).setRenderer(new myRenderer(this, myEventName));
			setRenderMode(RenderMode);
		}
	
		
		@Override
		@Hide
		public void innerInitialize(BA ba, String eventName, boolean keepOldObject)
		{
			if (!keepOldObject)
				setObject(new GLSurfaceView(ba.context));
			super.innerInitialize(ba, eventName, true);
		}

		/**
		 *Returns the API level supported by the device as an integer value.
		 *OpenGL2 is only supported by API level 8 (FROYO) devices or later.
		 */
		public int DeviceAPILevel()
		{
			return android.os.Build.VERSION.SDK_INT;
		}

		/**
		 *Causes the specified Sub to be scheduled for execution on the main GUI thread.
		 *This mechanism is the only way a thread can access GUI items. This call returns
		 *immediately. If it is necessary for the thread to wait for the result of the
		 *GUI operation then it should wait on a Lock that will be reset by the called Sub
		 *after it haas completed the GUI operation. Sometimes it seems that if the user
		 *presses the back button, and maybe in other scenarions, then Android loses the
		 *message and the GUI thread doesn't get it and the Sub isn't run and the Lock
		 *isn't freed so it may be advisable in this case to use WaitFor with a reasonably
		 *long timeout and retry until it returns True.
		 *
		 *Returns True if the Sub was found otherwise returns False.
		 */
		public boolean RunOnGuiThread(String sub, Object[] params)
		{
			sub = sub.toLowerCase();
			if (params == null)
				params = new Object[0];
			if (!ba.subExists(sub))
			{
				//error = ("RunOnGuiThread : Sub " + sub + " not found!");
				return false;
			}
			ba.raiseEventFromDifferentThread(this, this, ++taskId, sub, true, params);
			return true;
		}

		/**
		 *The renderer is called continuously to re-render the scene.
		 */
		public final int RENDERMODE_CONTINUOUSLY = GLSurfaceView.RENDERMODE_CONTINUOUSLY;
		/**
		 *The renderer only renders when the surface is created, or when requestRender() is called.
		 */
		public final int RENDERMODE_WHEN_DIRTY = GLSurfaceView.RENDERMODE_WHEN_DIRTY;

		/**
		 *Check glError() after every GL call and throw an exception if glError indicates that an error has occurred. This can be used to help track down which OpenGL ES call is causing an error.
		 */
		public final int DEBUG_CHECK_GL_ERROR = GLSurfaceView.DEBUG_CHECK_GL_ERROR;
		/**
		 *Log GL calls to the system log at "verbose" level with tag "GLSurfaceView".
		 */
		public final int DEBUG_LOG_GL_CALLS = GLSurfaceView.DEBUG_LOG_GL_CALLS;

		/**
		 *Get or set the debug flags to a new value.
		 *The value is constructed by OR-together zero or more of the DEBUG_CHECK_* constants.
		 *The debug flags take effect whenever a surface is created.
		 *The default value is zero.
		 */
		public int getDebugFlags()
		{
			return ((GLSurfaceView) getObject()).getDebugFlags();
		}

		public void setDebugFlags(int debugFlags)
		{
			((GLSurfaceView) getObject()).setDebugFlags(debugFlags);
		}

		/**
		 *Gets or sets the rendering mode.
		 *When renderMode is RENDERMODE_CONTINUOUSLY, the renderer is called repeatedly
		 *to re-render the scene. When renderMode is RENDERMODE_WHEN_DIRTY, the scene
		 *is only rendered when the surface is created, or when requestRender() is called.
		 *Defaults to RENDERMODE_CONTINUOUSLY.
		 *
		 *Using RENDERMODE_WHEN_DIRTY can improve battery life and overall system performance
		 *by allowing the GPU and CPU to idle when the view does not need to be updated.
		 *
		 *This method can only be called after setRenderer(Renderer)
		 */
		public int getRenderMode()
		{
			return ((GLSurfaceView) getObject()).getRenderMode();
		}

		public void setRenderMode(int renderMode)
		{
			((GLSurfaceView) getObject()).setRenderMode(renderMode);
		}

		/**
		 *Inform the view that the activity is paused.
		 *The owner of this view must call this method when the activity is paused.
		 *Calling this method will pause the rendering thread.
		 *Must not be called before a renderer has been set.
		 */
		public void Pause()
		{
			((GLSurfaceView) getObject()).onPause();
		}

		/**
		 *Queue a Sub to be run on the GL rendering thread.
		 *This can be used to communicate with the Renderer on the rendering thread and
		 *safely run non-atomic functions without any further thread synchronisation.
		 *Using this method to update data used while rendering ensures that the data will
		 *only be changed when rendering is not occurring,.
		 *If an uncaught exception occurs when running the event it is saved in the
		 *QueuedEventException property.
		 */
		public boolean QueueEvent(String eventname, final Object[] params)
		{
			final Method m = (Method) ba.processBA.htSubs.get(eventname.toLowerCase());
			if (m == null)
				return false;
			Runnable r = new Runnable()
			{
				@Override
				public void run()
				{
					try
					{
						m.invoke(ba.context, params);
					} catch (Exception ex)
					{
						e = ex;
					}
				};
			};
			((GLSurfaceView) getObject()).queueEvent(r);
			return true;
		}

		/**
		 *If an uncaught exception occurs when running a queued event it is saved here
		 *because an uncaught exception on a non-main thread will close the application.
		 */
		public Exception getQueuedEventException()
		{
			return e;
		}

		public void setQueuedEventException(Exception ex)
		{
			e = ex;
		}

		/**
		 *Inform the view that the activity is resumed.
		 *The owner of this view must call this method when the activity is resumed.
		 *Calling this method will recreate the OpenGL display and resume the rendering thread.
		 *Must not be called before a renderer has been set. 
		 */
		public void Resume()
		{
			((GLSurfaceView) getObject()).onResume();
		}

		/**
		 *Request that the renderer render a frame.
		 *This method is typically used when the render mode has been set to RENDERMODE_WHEN_DIRTY,
		 *so that frames are only rendered on demand.
		 *May be called from any thread.
		 *Must not be called before a renderer has been set. 
		 */
		public void RequestRender()
		{
			((GLSurfaceView) getObject()).requestRender();
		}


		private class myRenderer implements GLSurfaceView.Renderer
		{

			private String event;
			GLSurfaceViewWrapper2 glsv;

			public myRenderer(GLSurfaceViewWrapper2 glsv, String event)
			{
				this.event = event;
				this.glsv = glsv;
			}

			@Override
			public void onDrawFrame(GL10 gl)
			{
				// Ignore the passed-in GL10 interface, and use the GLES20
				// class's static methods instead.
				try
				{
					//Msgbox.isDismissing = true;
					ba.raiseEvent(glsv, event + "_draw", new Object[] { new GLWrapper2(gl) });
				} finally
				{
					//Msgbox.isDismissing = false;
				}
			}

			@Override
			public void onSurfaceChanged(GL10 gl, int width, int height)
			{
				// Ignore the passed-in GL10 interface, and use the GLES20
				// class's static methods instead.
				try
				{
					//Msgbox.isDismissing = true;
					ba.raiseEvent(glsv, event + "_surfacechanged", new Object[] { new GLWrapper2(gl), width, height });
				} finally
				{
					//Msgbox.isDismissing = false;
				}
			}

			@Override
			public void onSurfaceCreated(GL10 gl, EGLConfig config)
			{
				// Ignore the passed-in GL10 interface, and use the GLES20
				// class's static methods instead.
				try
				{
					//Msgbox.isDismissing = true;
					ba.raiseEvent(glsv, event + "_surfacecreated", new Object[] { new GLWrapper2(gl) });
				} finally
				{
					//Msgbox.isDismissing = false;
				}
			}
		}

	}

	/**
	 *This Matrix object  operates on OpenGL ES format matrices and vectors stored in float arrays.
	 *
	 *Matrices are 4 x 4 column-vector matrices stored in column-major order: 
	 *  m[offset +  0] m[offset +  4] m[offset +  8] m[offset + 12]
	 *  m[offset +  1] m[offset +  5] m[offset +  9] m[offset + 13]
	 *  m[offset +  2] m[offset +  6] m[offset + 10] m[offset + 14]
	 *  m[offset +  3] m[offset +  7] m[offset + 11] m[offset + 15]
	 * 
	 *Vectors are 4 row x 1 column column-vectors stored in order:
	 *  v[offset + 0] v[offset + 1] v[offset + 2] v[offset + 3]
	 *  
	 *More than one matrix or vector may be in an array. The offset parameter defines the location
	 *of the required item in the array.
	 */
	@ShortName("Matrix2")
	public static class MatrixWrapper2
	{

		/**
		 *Define a projection matrix in terms of six clip planes. 
		 *
		 *Parameters
		 *  m  the float array that holds the perspective matrix 
		 *  offset  the offset into float array m where the perspective matrix data is written 
		 */
		public void FrustumM(float[] m, int offset, float left, float right, float bottom, float top, float near, float far)
		{
			Matrix.frustumM(m, offset, left, right, bottom, top, near, far);
		}

		/**
		 *Inverts a 4 x 4 matrix into another matrix.
		 *
		 *Parameters
		 *  mInv  the array that holds the output inverted matrix 
		 *  mInvOffset  an offset into mInv where the inverted matrix is stored.
		 *  m  the input array
		 *  mOffset  an offset into m where the matrix is stored.
		 *  
		 *Returns
		 *  true if the matrix could be inverted, false if it could not. 
		 */
		public boolean InvertM(float[] mInv, int mInvOffset, float[] m, int mOffset)
		{
			return Matrix.invertM(mInv, mInvOffset, m, mOffset);
		}

		/**
		 *Computes the length of a vector.
		 */
		public float Length(float x, float y, float z)
		{
			return Matrix.length(x, y, z);
		}

		/**
		 *Multiply two 4x4 matrices together and store the result in a third 4x4 matrix.
		 *In matrix notation: result = lhs x rhs. Due to the way matrix multiplication works,
		 *the result matrix will have the same effect as first multiplying by the rhs matrix,
		 *then multiplying by the lhs matrix. This is the opposite of what you might expect.
		 *The same float array may be passed for result, lhs, and/or rhs.
		 *However, the result element values are undefined if the result elements overlap either
		 *the lhs or rhs elements.
		 *
		 *Parameters
		 *  result  The float array that holds the result.
		 *  resultOffset  The offset into the result array where the result is stored.
		 *  lhs  The float array that holds the left-hand-side matrix.
		 *  lhsOffset  The offset into the lhs array where the lhs is stored.
		 *  rhs  The float array that holds the right-hand-side matrix.
		 *  rhsOffset  The offset into the rhs array where the rhs is stored. 
		 */
		public void MultiplyMM(float[] result, int resultOffset, float[] lhs, int lhsOffset, float[] rhs, int rhsOffset)
		{
			Matrix.multiplyMM(result, resultOffset, lhs, lhsOffset, rhs, rhsOffset);
		}

		/**
		 *Multiply a 4 element vector by a 4x4 matrix and store the result in a 4 element column vector.
		 *In matrix notation: result = lhs x rhs The same float array may be passed for resultVec, lhsMat,
		 *and/or rhsVec. However, the resultVec element values are undefined if the resultVec elements
		 *overlap either the lhsMat or rhsVec elements.
		 *
		 *Parameters
		 *  resultVec  The float array that holds the result vector.
		 *  resultVecOffset  The offset into the result array where the result vector is stored.
		 *  lhsMat  The float array that holds the left-hand-side matrix.
		 *  lhsMatOffset  The offset into the lhs array where the lhs is stored.
		 *  rhsVec  The float array that holds the right-hand-side vector.
		 *  rhsVecOffset  The offset into the rhs vector where the rhs vector is stored. 
		 */
		public void MultiplyMV(float[] resultVec, int resultVecOffset, float[] lhsMat, int lhsMatOffset, float[] rhsVec,
				int rhsVecOffset)
		{
			Matrix.multiplyMV(resultVec, resultVecOffset, lhsMat, lhsMatOffset, rhsVec, rhsVecOffset);
		}

		/**
		 *Computes an orthographic projection matrix.
		 *
		 *Parameters
		 *  m  source matrix.
		 *  mOffset  index into m where the matrix starts.
		 *  a  angle to rotate in degrees.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.  
		 */
		public void OrthoM(float[] m, int mOffset, float left, float right, float bottom, float top, float near, float far)
		{
			Matrix.orthoM(m, mOffset, left, right, bottom, top, near, far);
		}

		/**
		 *Rotates matrix m in place by angle a (in degrees) around the axis (x, y, z).
		 *
		 *Parameters
		 *  rm  returns the result.
		 *  rmOffset  index into rm where the result matrix starts.
		 *  m  source matrix.
		 *  mOffset  index into m where the source matrix starts.
		 *  a  angle to rotate in degrees.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.  
		 */
		public void RotateM(float[] m, int mOffset, float a, float x, float y, float z)
		{
			Matrix.rotateM(m, mOffset, a, x, y, z);
		}

		/**
		 *Rotates matrix m by angle a (in degrees) around the axis (x, y, z) to another matrix.
		 *
		 *Parameters
		 *  rm  returns the result.
		 *  rmOffset  index into rm where the result matrix starts.
		 *  m  source matrix.
		 *  mOffset  index into m where the source matrix starts.
		 *  a  angle to rotate in degrees.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.  
		 */
		public void RotateM2(float[] rm, int rmOffset, float[] m, int mOffset, float a, float x, float y, float z)
		{
			Matrix.rotateM(rm, rmOffset, m, mOffset, a, x, y, z);
		}

		/**
		 *Scales matrix m by x, y, and z, putting the result in sm.
		 *
		 *Parameters
		 *  sm  returns the result.
		 *  smOffset  index into sm where the result matrix starts.
		 *  m  source matrix.
		 *  mOffset  index into m where the source matrix starts.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.
		 */
		public void ScaleM2(float[] sm, int smOffset, float[] m, int mOffset, float x, float y, float z)
		{
			Matrix.scaleM(m, mOffset, x, y, z);
		}

		/**
		 *Scales matrix m in place by x, y, and z.
		 *
		 *Parameters
		 *  m  matrix to scale.
		 *  mOffset  index into m where the matrix starts.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.  
		 */
		public void ScaleM(float[] m, int mOffset, float x, float y, float z)
		{
			Matrix.scaleM(m, mOffset, x, y, z);
		}

		/**
		 *Sets matrix m to the identity matrix.
		 *
		 *Parameters
		 *  m  returns the result.
		 *  mOffset  index into m where the result matrix starts. 
		 */
		public void SetIdentityM(float[] sm, int smOffset)
		{
			Matrix.setIdentityM(sm, smOffset);
		}

				
		/**
		 *Define a viewing transformation in terms of an eye point, a center of view, and an up vector.
		 *
		 *Parameters
		 *  rm returns the result
		 *  rmOffset  index into rm where the result matrix starts 
		 *  eyeX  eye point X 
		 *  eyeY  eye point Y 
		 *  eyeZ  eye point Z
		 *  centerX  center of view X
		 *  centerY  center of view Y
		 *  centerZ  center of view Z
		 *  upX  up vector X
		 *  upY  up vector Y
		 *  upZ  up vector Z
		 */
		public void SetLookAtM(float[] rm, int rmOffset, float eyeX, float eyeY, float eyeZ, float centerX, float centerY, float centerZ, float upX, float upY, float upZ)
		{
			Matrix.setLookAtM(rm, rmOffset, eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
		}
		
		/**
		 *Converts Euler angles x, y, z to a rotation matrix.
		 *
		 *Parameters
		 *  rm  returns the result.
		 *  rmOffset  index into rm where the result matrix starts.
		 *  x  angle of rotation, in degrees.
		 *  y  angle of rotation, in degrees.
		 *  z  angle of rotation, in degrees.  
		 */
		public void SetRotateEulerM(float[] rm, int rmOffset, float x, float y, float z)
		{
			Matrix.setRotateEulerM(rm, rmOffset, x, y, z);
		}

		/**
		 *Rotates matrix m by angle a (in degrees) around the axis (x, y, z).
		 *Parameters
		 *  rm  returns the result.
		 *  rmOffset  index into rm where the result matrix starts.
		 *  x  scale factor x.
		 *  y  scale factor y.
		 *  z  scale factor z.  
		 */
		public void SetRotateM(float[] rm, int rmOffset, float a, float x, float y, float z)
		{
			Matrix.setRotateM(rm, rmOffset, a, x, y, z);
		}

		/**
		 *Translates matrix m by x, y, and z in place.
		 *
		 *Parameters
		 *  m  matrix.
		 *  mOffset  index into m where the matrix starts.
		 *  x  translation factor x.
		 *  y  translation factor y.
		 *  z  translation factor z.  
		 */
		public void TranslateM(float[] m, int mOffset, float x, float y, float z)
		{
			Matrix.translateM(m, mOffset, x, y, z);
		}

		/**
		 *Translates matrix m by x, y, and z, putting the result in tm.
		 *
		 *Parameters
		 *  tm  returns the result.
		 *  tmOffset  index into sm where the result matrix starts.
		 *  m  source matrix.
		 *  mOffset  index into m where the source matrix starts.
		 *  x  translation factor x.
		 *  y  translation factor y.
		 *  z  translation factor z. 
		 */
		public void TranslateM2(float[] tm, int tmOffset, float[] m, int mOffset, float x, float y, float z)
		{
			Matrix.translateM(tm, tmOffset, m, mOffset, x, y, z);
		}

		/**
		 *Transposes a 4 x 4 matrix.
		 *
		 *Parameters
		 *  mTrans  the array that holds the output inverted matrix.
		 *  mTransOffset  an offset into mInv where the inverted matrix is stored.
		 *  m  the input array.
		 *  mOffset  an offset into m where the matrix is stored.  
		 */
		public void TransposeM(float[] mTrans, int mTransOffset, float[] m, int mOffset)
		{
			Matrix.transposeM(mTrans, mTransOffset, m, mOffset);
		}

	}

	/**
	 *A collection of utility methods for computing the visibility of triangle meshes. 
	 */
	@ShortName("Visibility2")
	public static class VisibilityWrapper2
	{

		/**
		 *Compute a bounding sphere for a set of points.
		 *It is approximately the minimal bounding sphere of an axis-aligned box that bounds the points.
		 *
		 *Parameters
		 *  positions  positions in x, y, z triples
		 *  positionsOffset  offset into positions array
		 *  positionsCount  number of position triples to process
		 *  sphere  array containing the output as (x, y, z, r)
		 *  sphereOffset offset where the sphere data will be written 
		 */
		public void computeBoundingSphere(float[] positions, int positionsOffset, int positionsCount, float[] sphere,
				int sphereOffset)
		{
			Visibility.computeBoundingSphere(positions, positionsOffset, positionsCount, sphere, sphereOffset);
		}

		/**
		 *Given an OpenGL ES ModelView-Projection matrix (which implicitly describes a frustum)
		 *and a list of spheres, determine which spheres intersect the frustum.
		 *
		 *A ModelView-Projection matrix can be computed by multiplying a Projection matrix by
		 *a ModelView matrix (in that order.). There are several possible ways to obtain the current
		 *ModelView and Projection matrices. The most generally applicable way is to keep track of the
		 *current matrices in application code. If that is not convenient, there are two optional
		 *OpenGL ES extensions which may be used to read the current matrices from OpenGL ES:
		 *
		 *GL10 - Ext.glQueryMatrixxOES
		 *GL11 - GL_MODELVIEW_MATRIX_FLOAT_AS_INT_BITS_OES and
		 *GL11 - GL_PROJECTION_MATRIX_FLOAT_AS_INT_BITS_OES
		 *
		 *The problem with reading back the matrices is that your application will only work with devices
		 *that support the extension(s) that it uses.
		 *
		 *A frustum is a six-sided truncated pyramid that defines the portion of world space that is visible in the view.
		 *
		 *Spheres are described as four floating point values: x, y, z, and r, in world-space coordinates.
		 *R is the radius of the sphere. 
		 *
		 *Parameters
		 *  mvp  a float array containing the mode-view-projection matrix 
		 *  mvpOffset  The offset of the mvp data within the mvp array. 
		 *  spheres  a float array containing the sphere data. 
		 *  spheresOffset  an offset into the sphere array where the sphere data starts
		 *  spheresCount  the number of spheres to cull.
		 *  results  an integer array containing the indices of the spheres that are either contained entirely within or intersect the frustum.
		 *  resultsOffset  an offset into the results array where the results start.
		 *  resultsCapacity  the number of array elements available for storing results.
		 *  
		 *Returns
		 *  the number of spheres that intersected the frustum. Can be larger than resultsCapacity,
		 *  in which case only the first resultsCapacity results are written into the results array.
		 */
		public int frustumCullSpheres(float[] mvp, int mvpOffset, float[] spheres, int spheresOffset, int spheresCount,
				int[] results, int resultsOffset, int resultsCapacity)
		{
			return Visibility.frustumCullSpheres(mvp, mvpOffset, spheres, spheresOffset, spheresCount, results, resultsOffset,
					resultsCapacity);
		}

		/**
		 *Test whether a given triangle mesh is visible on the screen.
		 *The mesh is specified as an indexed triangle list.
		 *
		 *Parameters
		 *  ws  the world space to screen space transform matrix, as an OpenGL column matrix.
		 *  wsOffset  an index into the ws array where the data starts.
		 *  positions  the vertex positions (x, y, z).
		 *  positionsOffset  the index in the positions array where the data starts.
		 *  indices  the indices of the triangle list. The indices are expressed as chars because they are unsigned 16-bit values.
		 *  indicesOffset  the index in the indices array where the index data starts.
		 *  indexCount  the number of indices in use. Typically a multiple of three. If not a multiple of three, the remaining one or two indices will be ignored.
		 *
		 *Returns
		 *  2 if all of the mesh is visible, 1 if some part of the mesh is visible, 0 if no part is visible.
		 */
		public int visibilityTest(float[] ws, int wsOffset, float[] positions, int positionsOffset, char[] indices,
				int indicesOffset, int indexCount)
		{
			return Visibility.visibilityTest(ws, wsOffset, positions, positionsOffset, indices, indicesOffset, indexCount);
		}

	}

}
